@@ -9,7 +9,7 @@ from django.conf import settings
 from django.template.defaultfilters import slugify
 from django.core.cache import cache
 
-from djangoplus.utils import path_to_object
+from djangoplus.utils import path_to_object, split1000
 from djangoplus import app_settings
 
 register = Library()
@@ -22,6 +22,10 @@ Boolean general utilities
 def multiple_of(value,arg):
     return value % arg == 0
 
+@register.filter
+def divide_by(value,arg):
+    return value / arg
+
 @register.filter
 def in_list(value,arg):
     if not value:
@@ -41,9 +45,13 @@ def as_string(value):
     # Returns unicode convertion of value
     return unicode(value)
 
+@register.filter
+def as_integer(value):
+    """Converts a number to integer"""
+    return int(value)
+
 @register.filter
 def is_not_equal(value,arg):
-    # O inverso do is_equal
     return value != arg
 
 @register.filter
@@ -154,12 +162,17 @@ def startswith(value,arg):
 def endswith(value,arg):
     return value.endswith(arg)
 
+@register.filter
+def replace(value,arg):
+    args = arg.split(',')
+    return value.replace(args[0], args[1])
+
 """
 Miscelanea
 """
 
 def multifind(s, q):
-    # Retorna uma lista de posicoes de uma string em outra - utilizado pelo highlight
+    # Returns a list of positions from a string in other one - used by 'highlight'
     ret = []
     sl = s.lower()
     q = q.lower()
@@ -177,7 +190,6 @@ def multifind(s, q):
 
 @register.filter
 def highlight(s, q):
-    # Destaca uma string dentro de outra em case insensitive, respeitando valor original
     pos = multifind(s, q)
 
     if not pos: return s
@@ -245,9 +257,12 @@ def make_cloud(object_list, count_attribute, steps=5):
     return [(obj, get_font_size(obj, steps_counts)) for obj in object_list]
 
 @register.filter_function
-def moneyformat(value, decimal_places=2):
+def moneyformat(value, decimal_places=2, thousands_separator=''):
     """Returns a float value in monetary format, using MONETARY_LOCALE
     setting and locale functions"""
+    decimal_places = decimal_places or 2
+    thousands_separator = thousands_separator or app_settings.THOUSANDS_SEPARATOR
+
     import locale
 
     if app_settings.MONETARY_LOCALE:
@@ -257,7 +272,14 @@ def moneyformat(value, decimal_places=2):
 
     value = value or 0.0
 
-    return locale.format(format, float(value), grouping=True)
+    ret = locale.format(format, float(value), grouping=True)
+
+    if thousands_separator:
+        dec_sep = thousands_separator == ',' and '.' or ','
+        parts = ret.split(dec_sep)
+        ret = split1000(parts[0], thousands_separator) + dec_sep + parts[1]
+
+    return ret
 
 @register.filter_function
 def has_module_perm(user, app_label):
@@ -400,12 +422,13 @@ class ChangeGetParamsNode(template.Node):
 
     def render(self, context):
         from urllib import urlencode
+
         # Resolve variables
         param = self.param.resolve(context)
         new_value = self.new_value.resolve(context)
 
         # Change current params
-        cur_params = dict([(k,v) for k,v in context['request'].GET.items()])
+        cur_params = dict([(k, v.encode('utf-8')) for k,v in context['request'].GET.items()])
         cur_params[param] = new_value
 
         return urlencode(cur_params)
@@ -435,7 +458,7 @@ class RemoveFromGetParamsNode(template.Node):
         param = self.param.resolve(context)
 
         # Change current params
-        cur_params = dict([(k,v) for k,v in context['request'].GET.items() if k != param])
+        cur_params = dict([(k, v.encode('utf-8')) for k,v in context['request'].GET.items() if k != param])
 
         return urlencode(cur_params)
 
@@ -631,3 +654,33 @@ def do_includefromurl(parser, token):
 
 register.tag('includefromurl', do_includefromurl)
 
+class AjaxFKWidgetCellsRender(template.Node):
+    vobj = None
+    vfields = None
+    var_result = None
+
+    def __init__(self, vobj, vfields, var_name):
+        self.vobj = template.Variable(vobj)
+        self.vfields = template.Variable(vfields)
+        self.var_result = var_name
+
+    def render(self, context):
+        obj = self.vobj.resolve(context)
+        list_display = self.vfields.resolve(context)
+
+        context[self.var_result] = [{'name': f, 'value': getattr(obj, f, None)} for f in list_display]
+
+        return ''
+
+def do_ajaxfkwidget_cells(parser, token):
+    try:
+        parts = token.split_contents()
+        vobj, vfields, var_name = parts[1:]
+    except IndexError, e:
+        raise template.TemplateSyntaxError, "%s requires 3 arguments" \
+                % token.contents.split()[0]
+
+    return AjaxFKWidgetCellsRender(vobj, vfields, var_name)
+
+register.tag('ajaxfkwidget_cells', do_ajaxfkwidget_cells)
+
